@@ -58,7 +58,7 @@ from . import matchers
 from .py37compat import best_realpath
 
 
-__all__ = ['Path', 'TempDir', 'CaseInsensitivePattern']
+__all__ = ['Path', 'TempDir']
 
 
 LINESEPS = ['\r\n', '\r', '\n']
@@ -93,28 +93,12 @@ def compose(*funcs):
     return functools.reduce(compose_two, funcs)
 
 
-def simple_cache(func):
-    """
-    Save results for the :meth:'path.using_module' classmethod.
-    When Python 3.2 is available, use functools.lru_cache instead.
-    """
-    saved_results = {}
-
-    def wrapper(cls, module):
-        if module in saved_results:
-            return saved_results[module]
-        saved_results[module] = func(cls, module)
-        return saved_results[module]
-
-    return wrapper
-
-
 class ClassProperty(property):
     def __get__(self, cls, owner):
         return self.fget.__get__(None, owner)()
 
 
-class multimethod(object):
+class multimethod:
     """
     Acts like a classmethod when invoked from the class and like an
     instancemethod when invoked from the instance.
@@ -131,6 +115,42 @@ class multimethod(object):
         )
 
 
+class Traversal:
+    """
+    Wrap a walk result to customize the traversal.
+
+    `follow` is a function that takes an item and returns
+    True if that item should be followed and False otherwise.
+
+    For example, to avoid traversing into directories that
+    begin with `.`:
+
+    >>> traverse = Traversal(lambda dir: not dir.startswith('.'))
+    >>> items = list(traverse(Path('.').walk()))
+
+    Directories beginning with `.` will appear in the results, but
+    their children will not.
+
+    >>> dot_dir = next(item for item in items if item.isdir() and item.startswith('.'))
+    >>> any(item.parent == dot_dir for item in items)
+    False
+    """
+
+    def __init__(self, follow):
+        self.follow = follow
+
+    def __call__(self, walker):
+        traverse = None
+        while True:
+            try:
+                item = walker.send(traverse)
+            except StopIteration:
+                return
+            yield item
+
+            traverse = functools.partial(self.follow, item)
+
+
 class Path(str):
     """
     Represents a filesystem path.
@@ -158,7 +178,7 @@ class Path(str):
             raise TypeError("Invalid initial value for path: None")
 
     @classmethod
-    @simple_cache
+    @functools.lru_cache
     def using_module(cls, module):
         subclass_name = cls.__name__ + '_' + module.__name__
         bases = (cls,)
@@ -292,11 +312,6 @@ class Path(str):
         base, ext = self.module.splitext(self.name)
         return base
 
-    @property
-    def namebase(self):
-        warnings.warn("Use .stem instead of .namebase", DeprecationWarning)
-        return self.stem
-
     @property
     def ext(self):
         """ The file extension, for example ``'.py'``. """
@@ -579,10 +594,12 @@ class Path(str):
             return
 
         for child in childList:
+            traverse = None
             if match(child):
-                yield child
+                traverse = yield child
+            traverse = traverse or child.isdir
             try:
-                isdir = child.isdir()
+                do_traverse = traverse()
             except Exception:
                 exc = sys.exc_info()[1]
                 tmpl = "Unable to access '%(child)s': %(exc)s"
@@ -590,7 +607,7 @@ class Path(str):
                 errors(msg)
                 isdir = False
 
-            if isdir:
+            if do_traverse:
                 for item in child.walk(errors=errors, match=match):
                     yield item
 
@@ -1312,7 +1329,6 @@ class Path(str):
         dst,
         symlinks=False,
         *,
-        update=False,
         copy_function=shutil.copy2,
         ignore=lambda dir, contents: []
     ):
@@ -1332,14 +1348,6 @@ class Path(str):
         dst = self._next_class(dst)
         dst.makedirs_p()
 
-        if update:
-            warnings.warn(
-                "Update is deprecated; " "use copy_function=only_newer(shutil.copy2)",
-                DeprecationWarning,
-                stacklevel=2,
-            )
-            copy_function = only_newer(copy_function)
-
         sources = self.listdir()
         _ignored = ignore(self, [item.name for item in sources])
 
@@ -1353,11 +1361,7 @@ class Path(str):
                 target.symlink(dest)
             elif source.isdir():
                 source.merge_tree(
-                    dest,
-                    symlinks=symlinks,
-                    update=update,
-                    copy_function=copy_function,
-                    ignore=ignore,
+                    dest, symlinks=symlinks, copy_function=copy_function, ignore=ignore,
                 )
             else:
                 copy_function(source, dest)
@@ -1542,7 +1546,7 @@ def only_newer(copy_func):
     return wrapper
 
 
-class SpecialResolver(object):
+class SpecialResolver:
     class ResolverScope:
         def __init__(self, paths, scope):
             self.paths = paths
@@ -1554,10 +1558,6 @@ class SpecialResolver(object):
     def __init__(self, path_class, *args, **kwargs):
         appdirs = importlib.import_module('appdirs')
 
-        # let appname default to None until
-        # https://github.com/ActiveState/appdirs/issues/55 is solved.
-        not args and kwargs.setdefault('appname', None)
-
         vars(self).update(
             path_class=path_class, wrapper=appdirs.AppDirs(*args, **kwargs)
         )
@@ -1610,12 +1610,16 @@ class TempDir(Path):
     constructed with the same parameters that you can use
     as a context manager.
 
-    Example::
+    For example:
 
-        with TempDir() as d:
-            # do stuff with the Path object "d"
+    >>> with TempDir() as d:
+    ...     d.isdir() and isinstance(d, Path)
+    True
 
-        # here the directory is deleted automatically
+    The directory is deleted automatically.
+
+    >>> d.isdir()
+    False
 
     .. seealso:: :func:`tempfile.mkdtemp`
     """
@@ -1644,10 +1648,6 @@ class TempDir(Path):
             self.rmtree()
 
 
-# For backwards compatibility.
-tempdir = TempDir
-
-
 def _multi_permission_mask(mode):
     """
     Support multiple, comma-separated Unix chmod symbolic modes.
@@ -1722,21 +1722,3 @@ def _permission_mask(mode):
         '=': lambda mask, target: target & retain ^ mask,
     }
     return functools.partial(op_map[op], mask)
-
-
-class CaseInsensitivePattern(matchers.CaseInsensitive):
-    def __init__(self, value):
-        warnings.warn(
-            "Use matchers.CaseInsensitive instead", DeprecationWarning, stacklevel=2
-        )
-        super(CaseInsensitivePattern, self).__init__(value)
-
-
-class FastPath(Path):
-    def __init__(self, *args, **kwargs):
-        warnings.warn(
-            "Use Path, as FastPath no longer holds any advantage",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        super(FastPath, self).__init__(*args, **kwargs)
